/***************************************************************************
    cdromdevice.cpp  -  CD-ROM device
    ------------------------------------------------------------------------
    begin                : Wed Aug 14 2002
    copyright            : (C) 2002, 2005 by Tomas Horsky
    email                : tomas@riadoklan.sk
 ***************************************************************************/

#include <kprocess.h>
#include <qregexp.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include "kcdssettings.h"

#include "cdromdevice.h"

CdromDevice::CdromDevice(const KcdsSettings *settings, QObject *parent, const char *name):
	QObject(parent, name),
	_settings(settings),
	_validDevice(false),
	_pOpened(false),
	_devFd(-1)
{
}

CdromDevice::~CdromDevice()
{
	if(_devFd>=0)
		closeDevice();
}

bool CdromDevice::persistentOpen(bool emitError)
{
	if(_pOpened)	//already opened persistently
		return true;
	if(_devFd>=0)	//make this persistent
		return _pOpened=true;
	return openDevice(emitError)? _pOpened=true: false;
}

bool CdromDevice::persistentClose()
{
	return closeDevice()? (_pOpened=false, true): false;
}

//check if device is opened, if not try to open, if unsuccessful return 'something'
#define OPEN(something) \
	if(_devFd<0 && !openDevice()) \
		return something

//close device if not opened persistently
#define CLOSE \
	if(!_pOpened) \
		closeDevice()

void CdromDevice::slotSelectSpeed(int speed)
{
	OPEN(;)
	if(ioctl(_devFd, CDROM_SELECT_SPEED, speed)<0)
		emit speedSelectError(speed);
	else
		emit speedSelected(speed);
	CLOSE;
}

void CdromDevice::slotEjectTray()
{
	if(_settings->ejectClose()!=KcdsSettings::Ioctl) {
		KProcess *proc=new KProcess;
		if(_settings->ejectClose()==KcdsSettings::Cdrecord) {
			if(_settings->scsiDevice()) {
				QString dev="dev=" +_settings->scsiBus() +"," +_settings->target() +"," +_settings->lun();
				*proc << "cdrecord" << dev << "-eject";
			} else
				*proc << "cdrecord" << "-eject";	//default device will be used
		} else {
			QString command=_settings->otherEject();
			interpretTokens(command);
			*proc << QStringList::split(" ", command);
		}
		connect(proc, SIGNAL(processExited(KProcess *)), SLOT(slotEjectProcessExited(KProcess *)));
		if(!proc->start())
			emit trayEjectError();
	} else {
		OPEN(;)
		if(ioctl(_devFd, CDROMEJECT)<0)
			emit trayEjectError();
		else
			emit trayEjected();
		CLOSE;
	}
}

void CdromDevice::slotCloseTray()
{
	if(_settings->ejectClose()!=KcdsSettings::Ioctl) {
		KProcess *proc=new KProcess;
		if(_settings->ejectClose()==KcdsSettings::Cdrecord) {
			if(_settings->scsiDevice()) {
				QString dev="dev=" +_settings->scsiBus() +"," +_settings->target() +"," +_settings->lun();
				*proc << "cdrecord" << dev << "-load";
			} else
				*proc << "cdrecord" << "-load";	//default device will be used
		} else {
			QString command=_settings->otherClose();
			interpretTokens(command);
			*proc << QStringList::split(" ", command);
		}
		connect(proc, SIGNAL(processExited(KProcess *)), SLOT(slotCloseProcessExited(KProcess *)));
		if(!proc->start())
			emit trayCloseError();
	} else {
		OPEN(;)
		if(ioctl(_devFd, CDROMCLOSETRAY)<0)
			emit trayCloseError();
		else
			emit trayClosed();
		CLOSE;
	}
}

int CdromDevice::getMaxSpeed() const
{
	return -1;
}

int CdromDevice::getCurrentSpeed() const
{
	return -1;
}

CdromDevice::Status CdromDevice::getStatus()
{
	OPEN(NoInfo);
	int stat=ioctl(_devFd, CDROM_DRIVE_STATUS);
	CLOSE;
	if(stat<0)
		return NoInfo;
	return (Status)stat;
}

bool CdromDevice::canSelectSpeed()
{
	OPEN(false);
	int cap=ioctl(_devFd, CDROM_GET_CAPABILITY);
	CLOSE;
	return (cap & CDC_SELECT_SPEED)==CDC_SELECT_SPEED? true: false;
}

bool CdromDevice::canEjectTray()
{
	OPEN(false);
	int cap=ioctl(_devFd, CDROM_GET_CAPABILITY);
	CLOSE;
	return (cap & CDC_OPEN_TRAY)==CDC_OPEN_TRAY? true: false;
}

bool CdromDevice::canCloseTray()
{
	OPEN(false);
	int cap=ioctl(_devFd, CDROM_GET_CAPABILITY);
	CLOSE;
	return (cap & CDC_CLOSE_TRAY)==CDC_CLOSE_TRAY? true: false;
}

bool CdromDevice::canLockTray()
{
	OPEN(false);
	int cap=ioctl(_devFd, CDROM_GET_CAPABILITY);
	CLOSE;
	return (cap & CDC_LOCK)==CDC_LOCK? true: false;
}

bool CdromDevice::testValidDevice(bool emitError)
{
	_pOpened=false;
	if(openDevice(emitError)) {
		closeDevice();
		return _validDevice=true;
	}
	return _validDevice=false;
}

int CdromDevice::speedToInt(const QString &speed)
{
	QString sp=speed;
	if(speed.right(1)=="x")
		sp.truncate(speed.length() -1);	//remove 'x'
	bool ok;
	int retval=sp.toInt(&ok);
	return ok? retval: -1;
}

QString CdromDevice::intToSpeed(int speed)
{
	QString sp;
	sp.setNum(speed);
	sp+="x";
	return sp;
}

bool CdromDevice::openDevice(bool emitError)
{
	if(_devFd>=0)
		closeDevice();
	QCString device=_settings->device().utf8();
	if((_devFd=::open(device, O_RDONLY | O_NONBLOCK))<0) {
		if(emitError)
			emit openDeviceError(device);
		return false;
	}
	return true;
}

bool CdromDevice::closeDevice()
{
	if(::close(_devFd)<0)	//global unistd.h close() !!!
		return false;
	_devFd=-1;
	return true;
}

void CdromDevice::interpretTokens(QString &s)
{
	s.replace(QRegExp("\\$DEVICE"), _settings->device());
	s.replace(QRegExp("\\$SCSIBUS"), _settings->scsiBus());
	s.replace(QRegExp("\\$TARGET"), _settings->target());
	s.replace(QRegExp("\\$LUN"), _settings->lun());
}

void CdromDevice::slotEjectProcessExited(KProcess *proc)
{
	if(proc->normalExit())
		emit trayEjected();
	else
		emit trayEjectError();
	delete proc;
}

void CdromDevice::slotCloseProcessExited(KProcess *proc)
{
	if(proc->normalExit())
		emit trayClosed();
	else
		emit trayCloseError();
	delete proc;
}
