/***************************************************************************
    cdromdevice.h  -  CD-ROM device
    ------------------------------------------------------------------------
    begin                : Wed Aug 14 2002
    copyright            : (C) 2002, 2005 by Tomas Horsky
    email                : tomas@riadoklan.sk
 ***************************************************************************/

#ifndef CDROMDEVICE_H
#define CDROMDEVICE_H

#include <qobject.h>
#include <qstring.h>
#include <linux/cdrom.h>

class KProcess;

class KcdsSettings;

/**	Representation of CD-ROM drive. It has pointer to KcdsSettings,	from
	which it takes all settings (path to device, etc...). Normally does not
	keep device opened, instead open()s and close()s it for every operation
	that it does with device file. persistentOpen() method provides way to
	open it persistently, do some work, and then persistentClose() device.
*/
class CdromDevice: public QObject
{
	Q_OBJECT
	public:
		enum Status {
			NoInfo=CDS_NO_INFO,
			NoDisc=CDS_NO_DISC,
			TrayOpen=CDS_TRAY_OPEN,
			DriveNotReady=CDS_DRIVE_NOT_READY,
			DiscOk=CDS_DISC_OK
		};
		
	public:
		/**	Constructor.
		*/
		CdromDevice(const KcdsSettings *settings, QObject *parent=0, const char *name=0);
		/**	Destructor.
		*/
		virtual ~CdromDevice();
		
		/**	Persistent open. Opens the device and does not close it until persistentClose()
			is called. If there are going to be several operations with CD-ROM
			(like calling all "bool can*Tray()" functions), this should be used to
			open the device, do the work, and close it by persistentClose().
			If the device is already opened, makes it persistent, instead of closing
			and reopening.
		*/
		bool persistentOpen(bool emitError=false);
		/**	Close persistently opened device.
		*/
		bool persistentClose();
		
	public slots:
		/**	Changes CD-ROM read speed.
		*/
		void slotSelectSpeed(int speed);
		/**	Ejects CD-ROM tray.
		*/
		void slotEjectTray();
		/**	Closes CD-ROM tray.
		*/
		void slotCloseTray();
		
	public:
		/**	NOT IMPLEMENTED
			Returns maximum read speed that device supports. If not supported,
			or if value is not available, returns -1.
		*/
		int getMaxSpeed() const;
		/**	NOT IMPLEMENTED
			Returns current selected speed. If not supported, or if value is
			not available, returns -1.
		*/
		int getCurrentSpeed() const;
		
		/**	Returns device status.
		*/
		Status getStatus();
		
		bool canSelectSpeed();
		bool canEjectTray();
		bool canCloseTray();
		bool canLockTray();
		
		bool isOpen() const {return _devFd>=0;}
		
		/**	Tries to open device, to see if path is valid, and if process has
			permissions to open it. Forces opening, so if CD-ROM is persistentOpen()ed,
			persistentClose()s it first. If successfull, returns true (but closes
			device). If not successfull, returns false, and if emitError is true
			also emits openDeviceError().
		*/
		bool testValidDevice(bool emitError=false);
		/**	Returns the result from last call to testValidDevice(). This means
			that the result is not changed until you call testValidDevice() again.
		*/
		bool isValidDevice() const {return _validDevice;}
		
		/**	Returns int value from speed string. Understands strings without
			"x" at the end, if it is number, it will be ok. If string is not
			a number at all, returns -1. Example: "40x" returns 40,
			"40" returns 40, "40a" returns -1...
		*/
		static int speedToInt(const QString &speed);
		/**	Returns speed string from int value. Example: 8 returns "8x".
		*/
		static QString intToSpeed(int speed);
		
	signals:
		void openDeviceError(const char *);
		void speedSelected(int);
		void speedSelectError(int);
		void trayEjected();
		void trayEjectError();
		void trayClosed();
		void trayCloseError();
		
	protected:
		bool openDevice(bool emitError=true);
		bool closeDevice();
		void interpretTokens(QString &s);
		
	protected slots:
		void slotEjectProcessExited(KProcess *proc);
		void slotCloseProcessExited(KProcess *proc);
		
	private:
		const KcdsSettings *_settings;
		bool _validDevice;
		bool _pOpened;
		int _devFd;
};

#endif //CDROMDEVICE_H
